function[] = networkMap(saveFile)
%% Plots the map of the proxy network and drought atlases
%
% networkMap
% Makes the plot
%
% networkMap(saveFile)
% Saves the plot to the specified file.
%
% ----- Inputs -----
% 
% saveFile: The name of the output file
%
% ----- Outputs -----
%
% Creates a .eps file

%%%%% Plot parameters %%%%%
latMax = -25;

pagesSizeLimits = [250, 25];
pagesDurationLimits = [2000, 100];
pagesYearLimits = [1, 2000];
pagesLegendSize = 10;
pagesColor = [.7 .7 .7];

atlasSize = 30;
atlasThickness = 1;
downgrid = "grid_20cr_cesm";

colormap = "redblue";
nColors = 20;
trimStrongColor = 4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Load and prepare data

% Load PAGES over the assimilation years
pages = gridfile('pages.grid').metadata;
times = pages.time>=pagesYearLimits(1) & pages.time<=pagesYearLimits(2);
[P, pages] = gridfile('pages.grid').load(["coord","time"], {[], times});

% Screen by latitude
[keep, pages] = screen.latitude(latMax, pages);
P(~keep,:) = [];

% Get the proxy type icon for each site. Use the duration of each record
% to determine icon size
icon = pagesMarkers(pages.attributes.type);
obs = ~isnan(P);
[~, first] = max(obs, [], 2);
[~, last] = max(fliplr(obs), [], 2);
last = pagesYearLimits(2) + 1 - last;
duration = last-first;
iconSize = interp1(pagesDurationLimits, pagesSizeLimits, duration);

% Sort icons by size so small icons are in front
[iconSize, sorted] = sort(iconSize, 'descend');
icon = icon(sorted);
pages.coord = pages.coord(sorted, :);

% Load the ANZDA and SADA coodinates
anzda = gridfile(sprintf('anzda-%s.grid',downgrid)).metadata.coord;
sada = gridfile(sprintf('sada-%s.grid',downgrid)).metadata.coord;

% Screen by latitude
anzda(anzda(:,1)>latMax, :) = [];
sada(sada(:,1)>latMax, :) = [];

% Load the instrumental field correlation
corrs = load('gong-corrs-20cr.mat');


%% Plot

% New map
figure('Position',[0 0 676.6666 639.3333]);
m_proj('stereo', 'rad', 90+latMax, 'lat', -90, 'lon', -120);
hold on

% Set the correlation background and coastline
m_contourg(corrs.lon, corrs.lat, corrs.rho);
m_coast;

% Drought atlases
m_scatter( sada(:,2), sada(:,1), atlasSize, 'x', 'MarkerEdgeColor', 'k', 'LineWidth', atlasThickness);
m_scatter( anzda(:,2), anzda(:,1), atlasSize, 'x', 'MarkerEdgeColor', 'k', 'LineWidth', atlasThickness);

% Add Pages sites
for s = 1:numel(icon)
    m_scatter( pages.coord(s,2), pages.coord(s,1), iconSize(s), pagesColor, ...
        icon(s), 'filled', 'MarkerEdgeColor', 'k');
end

% Get the colormap for the correlations and remove intense colors
cmap = cbrew(colormap, nColors+trimStrongColor, true);
trim = trimStrongColor/2;
cmap = cmap(1+trim:end-trim, :);

% Set colormap and colorbar
scaleColorMap(cmap, 0, gca, [-1 1]);
cb = colorbar('Location', 'south', 'AxisLocation', 'out');
cb.Position(2) = 0.05;

% Finalize the map
m_grid('XaxisLocation', 'top', 'YaxisLocation', 'middle', 'fontsize',14);
set(gca,'fontsize',16);
ax = gca;
ax.Position(2) = 0.13;


%% PAGES Legend

% Get PAGES types and icons
types = [ "Glacier Ice", "Lake Sediment", "Tree", "Coral"];
typeIcon = ["o","s","v","d"];

% Preallocate handles to invisible icons
nType = numel(types);
nEmpty = 4;
ph = NaN(nType + nEmpty + 1, 1);

% Plot invisible proxy type icons
ph(1) = scatter(NaN, NaN, atlasSize, 'kx', 'MarkerEdgeColor', 'k', 'LineWidth', atlasThickness);
for t = 1:nType
%     ph(t+1) = scatter(NaN, NaN, pagesLegendSize, pagesColor, typeIcon(t), 'filled', 'MarkerEdgeColor', 'k');
    ph(t+1) = plot(NaN, NaN, typeIcon(t), 'MarkerFaceColor', pagesColor, 'MarkerSize', pagesLegendSize, 'MarkerEdgeColor','k');
end
for k = 1:nEmpty
    ph(end-k+1) = plot(NaN, NaN, 'w');
end

% Create the initial legend of type icons. Include space for size markers
labels = ["Drought Atlas", types(1:2), "", "", types(3:4), "", ""];
ph = ph([1:3, 6:7, 4:5, 8:9]);
l1 = legend(ph, labels, 'Location', 'northwest','NumColumns',2);
l1.Position = [0.1113 0.72162 0.40788 0.19708];

% Create invisible axis to hold size icon markers
ax1 = gca;
ax2 = axes('Position', ax1.Position);
hold on

% Get handles for icon size markers
p1 = plot(NaN, NaN, "v", 'MarkerSize', 5, 'MarkerFaceColor', pagesColor, 'MarkerEdgeColor', 'k');
p2 = plot(NaN, NaN, "v", 'MarkerSize', 13, 'MarkerFaceColor', pagesColor, 'MarkerEdgeColor', 'k');

% Create second legend
l2 = legend([p1 p2], "100 years", "2,000 years", 'NumColumns', 2);
set(ax2, 'Fontsize',16);
l2.Box = 'off';

% Make the second axis invisible and move legend 2 into legend 1
ax2.Visible = 'off';
l2.Position = [0.1258    0.7425    0.3901    0.0396];

% Save to .eps
if nargin>0
    print(saveFile, '-depsc', '-painters');
end

end