function[] = sampleDepth(saveFile)
%% Makes the sample depth figure
%
% sampleDepth
% Plot the figure
%
% sampleDepth(saveFile)
% Save the figure to file
%
% ----- Inputs -----
%
% saveFile: The name of the output file
%
% ----- Outputs -----
%
% Creates an .eps file

%%%%% Plot parameters %%%%%
maxLat = -25;
downgrid = "grid_20cr_cesm";
years = 1:2000;
networks = ["pages","anzda","sada"];
%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Select colors in the colormap
colors = [
    .6597 0.0818 0.1625    % 1 Dark Red
    .1151, .3724, .6361    % 2 Dark blue  
    .4513, .6958, .8292    % 3a Light blue
%     .5833, .7780, .8737    % 3b Lighter blue
    .9108, .5412, .4285    % 4a Light Red
%     .9584, .6563, .5216    % 4b Lighter red
    0, 0, 0                % 5 Black
    .8 .8 .8               % 6 Light grey
    ];

% Set the order of proxies and associated colors
proxyType = ["lake sediment","glacier ice","tree","sada","anzda","coral"];
colorIndex = [5 3 1 6 4 2];

% Get the proxy observations and screen by latitude
[Y, meta] = load.observations(networks, years, downgrid);
[keep, meta] = screen.latitude(maxLat, meta);
Y = Y(keep,:);
obs = ~isnan(Y);

% Get the proxy type for each site
type = da.siteNames(networks, meta);
type(~ismember(type, ["sada","anzda"])) = meta{networks=="pages"}.attributes.type;
type(type=="borehole") = "glacier ice";

% Get the types available over time
type = repmat(type, [1, numel(years)]);
type(~obs) = "";

% Get the size of each color in each time step
nType = numel(proxyType);
nTime = numel(years);
nProxy = NaN(nType, nTime);
for t = 1:nType
    nProxy(t,:) = sum(type==proxyType(t), 1);
end

% Smooth fuzzy proxies to make plot readable
for t = 2:nTime-1
    prevLarger = any(nProxy(:,1:t-1) > nProxy(:,t), 2);
    futureLarger = any(nProxy(:,t+1:end) > nProxy(:,t), 2);
    smooth = prevLarger & futureLarger;
    nProxy(smooth, t) = max( nProxy(smooth, 1:t-1), [], 2);
end
% nProxy(strcmp(proxyType, 'glacier ice'), daYears==1999) = 10;

% Calculate the edges of each patch
top = cumsum(nProxy);
bottom = fliplr( [zeros(1, nTime); top(1:end-1,:)] );
x = years(:);
x = [x; flipud(x)];

% Preallocate patch handles
phand = NaN(nType, 1);

% Plot each patch and the total number of proxies
figure('Position', [1.6667 199 1044 310.6667]);
hold on
for c = 1:nType
    k = colorIndex(c);
    phand(c) = patch(x, [top(c,:), bottom(c,:)], colors(k,:), 'Edgecolor', 'none');
end
ptot = plot(years, top(end,:), 'k', 'linewidth', 1);

% Legend
labels = {'Total', 'Coral', 'ANZDA', 'SADA', 'Trees', 'Glacier Ice', 'Lake Sediment'};
legend([ptot; flipud(phand)], labels, 'Location', 'northwest','NumColumns',2);

% Finalize
set(gca,'fontsize',21,'TickDir','out','YAxisLocation','right');
xlabel('Year (CE)');
ylabel('  Number of Proxies', 'Rotation', 270, 'VerticalAlignment', 'bottom');
xticks(0:100:2000);
labels = strings(1,21);
labels([1,6,11,16,21]) = ["0","500","1000","1500","2000"];
xticklabels(labels);
set(gca, 'XTickLabelRotation', 0);

% Save
if nargin>0
    print(saveFile, '-depsc', '-painters');
end

end