function[rho, pval, priors] = O21statistics
%% O21statistics  Computes correlation and p-value of O21 with the Marshall index
% ----------
%   [rho, pval, priors] = O21statistics
%   Returns correlations and p-values for the O'Connor et al., 2021 
%   reconstructions. (O21 reports skill on a different time interval than
%   this study. We recalculate statistics on our own time interval to
%   enable a fair comparison.) Reported p-values use an effective sample
%   size accounting for AR1.
% ----------
%   Outputs:
%       rho (numeric vector [4]): Correlations with the Marshall index
%       pval (numeric vector [4]): AR1 adjusted p-values
%       priors (string vector [4]): The ID of the prior associated with
%           each rho and p-value

% Get the reconstruction and Marshall index
o21 = gridfile('oconnor-2021.grid');
marsh = gridfile('marshall-djf.grid');

% Get the priors in O21
priors = o21.metadata.run;
nPrior = numel(priors);

% Limit to skill interval
years = parameters.years.skill;
oYears = ismember(o21.metadata.time, years);
mYears = ismember(marsh.metadata.time, years);

% Load the reconstruction and marshall index
o21 = o21.load("time", {oYears});
marsh = marsh.load("time", {mYears});

% Get statistics for each reconstruction
rho = NaN(nPrior, 1);
pval = NaN(nPrior, 1);
for p = 1:nPrior
    [rho(p), pval(p)] = brethertonCorr(o21(:,p), marsh);
end

end