function[cmap] = scaleMap( cmap, x0, clim, ax, noCenterFlag )
%% Scales a colormap about a centering value.
%
% cmap = scaleMap( map, x0 )
% Scales a colormap about centering value x0 for the data on the current
% axis.
%
% cmap = scaleMap( map, x0, ax )
% Scales a colormap for the data on a specified axis.
%
% cmap = scaleMap( map, x0, clim )
% Scales a colormap for specified color limits. If clim = [], then data is
% scaled to the color limits of the current axis.
%
% cmap = scaleMap( map, x0, clim, ax )
% Also sets the color limits of the specified axis to clim. If ax = [],
% then the color limits are not altered for any axis.
%
% cmap = scaleMap( map, x0, ..., ..., 'default' )
% Throws an error if the color limits do not contain the centering value.
%
% cmap = scaleMap( map, x0, ..., ..., 'halfmap' )
% If the color limits do not contain the centering value, uses a full half
% of the colormap and sets x0 as one color limits, as appropriate.
%
% cmap = scaleMap( map, x0, ..., ..., 'scalehalf' )
% If the color limits do not contain the centering value, scales one half
% of the colormap and does not alter the color limits.
%
%
% ----- Inputs -----
%
% map: A color map
%
% x0: The centering value for the map
%
% ax: An axis handle
%
% clim: Color limits. A 2-element numeric vector.
%
%
% ----- Outputs -----
%
% cmap: The centered, scaled map.

% ----- Written By -----
% Jonathan King, University of Arizona, 2019

% Get the color limits
if nargin < 3
    clim = get( gca, 'clim' );
elseif isa(ax, 'matlab.graphics.axis.Axes')
    clim = get( ax, 'clim' );
elseif isvector(ax) && numel(ax)==2
    clim = ax;
else
    error('Unrecognized third input.');
end


% Do some setup

% Get the deviation of each color limit from the centering value
dev = clim - x0;

% Check if the centering value is within the color limits

% Ensure that the centering value is within the color limits
if all( dev<0 ) || all(dev>0) || all(dev==0)
    error('The centering value is not within the color limits.');
end

% Get the absolute deviation of each color limit
dev = abs( dev );

% Get the percent of the maximum deviation associated with each limit
percDev = dev ./ max(dev);

% Get the number of color points on each side of the colormap center
halfStep = floor( size(cmap,1) / 2 );

% Trim the bottom (maximum values) of the colormap
nTrim = halfStep - round( percDev(2)*halfStep );
cmap( end-nTrim+1:end, : ) = [];

% Trim the top (minimum values) of the colormap
nTrim = halfStep - round( percDev(1)*halfStep );
cmap( 1:nTrim, : ) = [];

end 