function [smoothed,w] = gaussiansmooth(x,p,pad)

% GAUSSIAN SMOOTH Apply a Gaussian smooth to a time series
%
%   [smoothed,w] = gaussiansmooth(x,p,pad)
%                 
% x = time series data vector (m x 1) to be smoothed
% p = period of the smoothing to be applied
% pad = how the ends of the series should be padded (currently 'none' or 'mean')

% make column vector
x = x(:); 
                    
% default is no padding
if nargin < 3
 pad = 'none';
end

original = x;
not_missing = ~isnan(x);
x = x(not_missing);

% apply padding         
switch pad
 case 'none'
  x = x;
 case 'mean'
  x = [ones(p,1)*nanmean(x(1:p),1); x; ones(p,1)*nanmean(x(end-p+1:end),1)];
end

% get weights
w = wtsgaus(p,length(x)); 

% filter series with weights
smoothed = filtfilt(w,1,x);

% remove padding if applied
switch pad
 case 'none'
  smoothed = smoothed;
 case 'mean'
  smoothed = smoothed(p+1:end-p);
end 

% reinsert into smoothed for output
temp = NaN(size(original));
temp(not_missing) = smoothed;
smoothed = temp;


%% Function to get the Gaussian weights for the filter
function w = wtsgaus(p,N)
% WTSGAUS weights for gaussian filter with 50% response at period of p years

if p > N; 
    error(['Desired 50% period ' num2str(p) ' is greater than  the sample length ' int2str(N)]);
end

% Check that period of 50% response at least 5 yr
if p < 5;
  error('Period of 50% response must be at least 5 yr');
end

sigma = p/6;  % Gaussian curve should have this standard deviation

x     = -N:N;
w     = normpdf(x/sigma,0,1);
wmax  = max(w);
wkeep = w >= 0.05*wmax; % keep weights at least 5% as big as central weight
w     = w(wkeep);
w     = w/sum(w); % force weights to sum to one


